@@ -2,11 +2,12 @@ from __future__ import annotations
 
 import collections
 import pathlib
+from typing import TYPE_CHECKING
 
 from import_me.columns import Column
 from import_me.exceptions import ColumnError, ParserError, SkipRow, StopParsing
 
-if False:  # TYPE_CHECKING
+if TYPE_CHECKING:
     from typing import List, Dict, Tuple, Any, Type, Union, IO, Iterator, DefaultDict
 
 
@@ -69,8 +70,10 @@ class BaseParser(ParserMixin):
 
         self.cleaned_data = self.clean(data)
 
-    def parse_row(self, row: List[Any], row_index: int) -> Dict:
+    def parse_row(self, row: List[Any], row_index: int, worksheet_title: str = None) -> Dict:
         row_data = {}
+        if worksheet_title:
+            row_data['worksheet'] = worksheet_title
         row_has_errors = False
 
         for column in self.columns:
@@ -78,12 +81,17 @@ class BaseParser(ParserMixin):
                 row_data[column.name] = self.parse_column(row, column, row_index)
             except ColumnError as e:
                 row_has_errors = True
-                self.add_errors(e.messages, row_index=row_index, col_index=column.index)
+                self.add_errors(
+                    e.messages,
+                    row_index=row_index,
+                    col_index=column.index,
+                    worksheet_title=worksheet_title,
+                )
 
         if row_has_errors:
             raise SkipRow('Not processed because the string contains errors.')
 
-        return self.clean_row(row_data, row, row_index)
+        return self.clean_row(row_data, row, row_index, worksheet_title=worksheet_title)
 
     def parse_column(self, row: List[Any], column: Column, row_index: int) -> Any:
         try:
@@ -102,12 +110,12 @@ class BaseParser(ParserMixin):
 
         return value
 
-    def clean_row(self, row_data: Dict, row: List[Any], row_index: int) -> Dict:
+    def clean_row(self, row_data: Dict, row: List[Any], row_index: int, worksheet_title: str = None) -> Dict:
         if self.skip_empty_rows and all((row_data.get(column.name) is None for column in self.columns)):
             raise SkipRow
 
-        row_data = self.clean_row_required_columns(row_data, row, row_index)
-        row_data = self.clean_unique_together_values(row_data, row, row_index)
+        row_data = self.clean_row_required_columns(row_data, row, row_index, worksheet_title=worksheet_title)
+        row_data = self.clean_unique_together_values(row_data, row, row_index, worksheet_title=worksheet_title)
 
         if self.add_file_path:
             row_data['file_path'] = self.file_path
@@ -116,14 +124,16 @@ class BaseParser(ParserMixin):
 
         return row_data
 
-    def clean_row_required_columns(self, row_data: Dict, row: List[Any], row_index: int) -> Dict:
+    def clean_row_required_columns(
+        self, row_data: Dict, row: List[Any], row_index: int, worksheet_title: str = None,
+    ) -> Dict:
         has_empty_required_columns = False
 
         for column in self.columns:
             if column.required and row_data.get(column.name) is None:
                 self.add_errors(
                     f'Column {column.header or column.name} is required.',
-                    row_index=row_index, col_index=column.index,
+                    row_index=row_index, col_index=column.index, worksheet_title=worksheet_title,
                 )
                 has_empty_required_columns = True
 
@@ -132,7 +142,9 @@ class BaseParser(ParserMixin):
 
         return row_data
 
-    def clean_unique_together_values(self, row_data: Dict, row: List[Any], row_index: int) -> Dict:
+    def clean_unique_together_values(
+        self, row_data: Dict, row: List[Any], row_index: int, worksheet_title: str = None,
+    ) -> Dict:
         is_not_unique_row = False
 
         if not self._unique_together:
@@ -153,7 +165,7 @@ class BaseParser(ParserMixin):
                     ))
                     self.add_errors(
                         f'{error} is a duplicate of row {duplicate_row}',
-                        row_index=row_index,
+                        row_index=row_index, worksheet_title=worksheet_title,
                     )
                     is_not_unique_row = True
                 else:
@@ -182,11 +194,19 @@ class BaseParser(ParserMixin):
     def clean(self, data: List) -> List:
         return data
 
-    def add_errors(self, messages: Union[str, List], row_index: int = None, col_index: int = None) -> None:
+    def add_errors(
+        self,
+        messages: Union[str, List],
+        row_index: int = None,
+        col_index: int = None,
+        worksheet_title: str = None,
+    ) -> None:
         if not isinstance(messages, list):
             messages = [messages]
         for message in messages:
             error = []
+            if worksheet_title is not None:
+                error.append(f'worksheet: {worksheet_title}')
             if row_index is not None:
                 error.append(f'row: {row_index}')
             if col_index is not None:
